﻿using System;
using System.Linq;
using System.Linq.Expressions;
using System.Collections.Generic;

namespace Achiral.Extension
{
    public static partial class MakeWorld
    {
        public static IEnumerable<T> Unfold<T>(this Func<T, T> func, T initial)
        {
            AssertNonNullArgument(func, "func");
            var value = initial;
            yield return value;
            while (true)
            {
                value = func(value);
                yield return value;
            }
        }

        public static Func<T1, TResult> Unbind1st<T1, TResult>(this Func<TResult> func, T1 dummy)
        {
            AssertNonNullArgument(func, "func");

            return func.Method.CreateDelegate<Func<T1, TResult>>();
        }
        public static Func<T1, T2, TResult> Unbind1st<T1, T2, TResult>(this Func<T2, TResult> func, T1 dummy)
        {
            AssertNonNullArgument(func, "func");

            return func.Method.CreateDelegate<Func<T1, T2, TResult>>();
        }
        public static Func<T1, T2, T3, TResult> Unbind1st<T1, T2, T3, TResult>(this Func<T2, T3, TResult> func, T1 dummy)
        {
            AssertNonNullArgument(func, "func");

            return func.Method.CreateDelegate<Func<T1, T2, T3, TResult>>();
        }
        public static Func<T1, T2, T3, T4, TResult> Unbind1st<T1, T2, T3, T4, TResult>(this Func<T2, T3, T4, TResult> func, T1 dummy)
        {
            AssertNonNullArgument(func, "func");

            return func.Method.CreateDelegate<Func<T1, T2, T3, T4, TResult>>();
        }

        public static TResult InvokeOrDefault<TResult>(this Func<TResult> func)
        {
            if (func == null) return default(TResult);

            return func();
        }
        public static TResult InvokeOrDefault<T, TResult>(this Func<T, TResult> func, T arg)
        {
            if (func == null) return default(TResult);

            return func(arg);
        }
        public static TResult InvokeOrDefault<T1, T2, TResult>(this Func<T1, T2, TResult> func, T1 arg1, T2 arg2)
        {
            if (func == null) return default(TResult);

            return func(arg1, arg2);
        }
        public static TResult InvokeOrDefault<T1, T2, T3, TResult>(this Func<T1, T2, T3, TResult> func, T1 arg1, T2 arg2, T3 arg3)
        {
            if (func == null) return default(TResult);

            return func(arg1, arg2, arg3);
        }
        public static TResult InvokeOrDefault<T1, T2, T3, T4, TResult>(this Func<T1, T2, T3, T4, TResult> func, T1 arg1, T2 arg2, T3 arg3, T4 arg4)
        {
            if (func == null) return default(TResult);

            return func(arg1, arg2, arg3, arg4);
        }

        public static IEnumerable<T> CascadeDepthFirst<T>(this Func<T, IEnumerable<T>> func, T initial)
        {
            AssertNonNullArgument(func, "func");

            return Make.Sequence(initial).CascadeDepthFirst(func);
        }
        public static IEnumerable<TResult> CascadeDepthFirst<T, TResult>(this Func<T, IEnumerable<T>> func, T initial, Func<T, int, TResult> selector)
        {
            AssertNonNullArgument(func, "func");

            return Make.Sequence(initial).CascadeDepthFirst(func, selector);
        }
        public static IEnumerable<T> CascadeBreadthFirst<T>(this Func<T, IEnumerable<T>> func, T initial)
        {
            AssertNonNullArgument(func, "func");

            return Make.Sequence(initial).CascadeBreadthFirst(func);
        }
        public static IEnumerable<TResult> CascadeBreadthFirst<T, TResult>(this Func<T, IEnumerable<T>> func, T initial, Func<T, int, TResult> selector)
        {
            AssertNonNullArgument(func, "func");

            return Make.Sequence(initial).CascadeBreadthFirst(func, selector);
        }
    }
}