﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;

namespace Achiral.Extension
{
    public static partial class MakeWorld
    {
        public static IEnumerable<T> CascadeDepthFirst<T>(this ILookup<T, T> lookup, T initial)
        {
            AssertNonNullArgument(lookup, "lookup");

            return Make.Func((T item) => lookup[item]).CascadeDepthFirst(initial);
        }
        public static IEnumerable<TResult> CascadeDepthFirst<T, TResult>(this ILookup<T, T> lookup, T initial, Func<T, int, TResult> selector)
        {
            AssertNonNullArgument(lookup, "lookup");
            AssertNonNullArgument(selector, "selector");

            return Make.Func((T item) => lookup[item]).CascadeDepthFirst(initial, selector);
        }
        public static IEnumerable<T> CascadeBreadthFirst<T>(this ILookup<T, T> lookup, T initial)
        {
            AssertNonNullArgument(lookup, "lookup");

            return Make.Func((T item) => lookup[item]).CascadeBreadthFirst(initial);
        }
        public static IEnumerable<TResult> CascadeBreadthFirst<T, TResult>(this ILookup<T, T> lookup, T initial, Func<T, int, TResult> selector)
        {
            AssertNonNullArgument(lookup, "lookup");
            AssertNonNullArgument(selector, "selector");

            return Make.Func((T item) => lookup[item]).CascadeBreadthFirst(initial, selector);
        }

        public static Func<TKey, IEnumerable<TElement>> AsFunc<TKey, TElement>(this ILookup<TKey, TElement> lookup)
        {
            AssertNonNullArgument(lookup, "lookup");

            return key => lookup[key];
        }
    }

}