﻿using System;
using System.Linq;
using System.Reflection;

namespace Achiral.Extension
{
    public static partial class MakeWorld
    {
        public static ConstructorInfo[] GetAllConstructors(this Type type)
        {
            return type.GetAllConstructors(false);
        }
        public static ConstructorInfo[] GetAllConstructors(this Type type, bool declaredOnly)
        {
            AssertNonNullArgument(type, "type");
            return type.GetConstructors(GetBindingFlagsForAllElement(declaredOnly));
        }
        public static FieldInfo[] GetAllFields(this Type type)
        {
            return type.GetAllFields(false);
        }
        public static FieldInfo[] GetAllFields(this Type type, bool declaredOnly)
        {
            AssertNonNullArgument(type, "type");
            return type.GetFields(GetBindingFlagsForAllElement(declaredOnly));
        }
        public static MemberInfo[] GetAllMembers(this Type type)
        {
            return type.GetAllMembers(false);
        }
        public static MemberInfo[] GetAllMembers(this Type type, bool declaredOnly)
        {
            AssertNonNullArgument(type, "type");
            return type.GetMembers(GetBindingFlagsForAllElement(declaredOnly));
        }
        public static MethodInfo[] GetAllMethods(this Type type)
        {
            return type.GetAllMethods(false);
        }
        public static MethodInfo[] GetAllMethods(this Type type, bool declaredOnly)
        {
            AssertNonNullArgument(type, "type");
            return type.GetMethods(GetBindingFlagsForAllElement(declaredOnly));
        }
        public static PropertyInfo[] GetAllProperties(this Type type)
        {
            return type.GetAllProperties(false);
        }
        public static PropertyInfo[] GetAllProperties(this Type type, bool declaredOnly)
        {
            AssertNonNullArgument(type, "type");
            return type.GetProperties(GetBindingFlagsForAllElement(declaredOnly));
        }
        private static BindingFlags GetBindingFlagsForAllElement(bool declaredOnly)
        {
            var flags = BindingFlags.Static
                      | BindingFlags.Instance
                      | BindingFlags.NonPublic
                      | BindingFlags.Public;
            flags = declaredOnly ? flags | BindingFlags.DeclaredOnly : flags;
            return flags;
        }
    }
}