﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Achiral;
using Achiral.Extension;
using System.IO;

namespace Achiral.Test
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class MakeTest
    {
        public MakeTest()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext { get; set; }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void MakeMethodInfo_Test()
        {
            var methodInfo1 = typeof(int).GetMethod("CompareTo", new[] { typeof(object) });
            var methodInfo2 = Make.MethodInfo(() => (default(int)).CompareTo((default(object))));

            Assert.AreEqual(methodInfo1, methodInfo2);
        }

        [TestMethod]
        public void MakeDictionary_Test()
        {
            var dict1 = new Dictionary<string, string>()
            {
                {"x", "foo"},
                {"y", "bar"},
                {"Hauhau", "gaogao"}
            };

            var dict2 = Make.Dictionary
            (
                x => "foo",
                y => "bar",
                Hauhau => "gaogao"
            );

            Assert.IsTrue(Enumerable.SequenceEqual(dict1, dict2));
        }

        [TestMethod]
        public void MakeFix_Test()
        {
            var fact1 = default(Func<int, int>);
            fact1 = x => x == 0 ? 1 : x * fact1(x - 1);

            var fact2 = Make.Fix<int>(fac => x => x == 0 ? 1 : x * fac(x - 1));

            const int count = 10;
            Enumerable.Range(1, count).ForEach(i => Assert.AreEqual(fact1(i), fact2(i)));
        }

        [TestMethod]
        public void MakeExpression_FuncTest1()
        {
            var expr = Make.Expression((int a) => a);

            var result1 = expr.Bind1st(3).Compile()();

            Assert.AreEqual(3, result1, "Bind1st");
        }

        [TestMethod]
        public void MakeExpression_FuncTest2()
        {
            var expr = Make.Expression((int a, int b) => a * b);

            var result1 = expr.Bind1st(3).Bind1st(4).Compile()();
            var result2 = expr.Bind2nd(4).Bind1st(3).Compile()();

            Assert.AreEqual(12, result1, "Bind1st");
            Assert.AreEqual(12, result2, "Bind2nd");
        }

        [TestMethod]
        public void MakeExpression_FuncTest3()
        {
            var expr = Make.Expression((int a, int b, int c) => a * b % c);

            var result1 = expr.Bind1st(3).Bind1st(4).Bind1st(7).Compile()();
            var result2 = expr.Bind2nd(4).Bind1st(3).Bind1st(7).Compile()();
            var result3 = expr.Bind3rd(7).Bind1st(3).Bind1st(4).Compile()();

            Assert.AreEqual(5, result1, "Bind1st");
            Assert.AreEqual(5, result2, "Bind2nd");
            Assert.AreEqual(5, result3, "Bind3rd");
        }

        [TestMethod]
        public void MakeExpression_FuncTest4()
        {
            var expr = Make.Expression((int a, int b, int c, int d) => a * b % c + d);

            var result1 = expr.Bind1st(3).Bind1st(4).Bind1st(7).Bind1st(1).Compile()();
            var result2 = expr.Bind2nd(4).Bind1st(3).Bind1st(7).Bind1st(1).Compile()();
            var result3 = expr.Bind3rd(7).Bind1st(3).Bind1st(4).Bind1st(1).Compile()();
            var result4 = expr.Bind4th(1).Bind1st(3).Bind1st(4).Bind1st(7).Compile()();

            Assert.AreEqual(6, result1, "Bind1st");
            Assert.AreEqual(6, result2, "Bind2nd");
            Assert.AreEqual(6, result3, "Bind3rd");
            Assert.AreEqual(6, result4, "Bind4th");
        }


        [TestMethod]
        public void MakeExpression_ActionTest1()
        {
            var result = default(int);
            var setResult = Make.Action((int i) => result = i);
            var expr = Make.Expression((int a) => setResult(a));

            expr.Bind1st(3).Compile()();

            Assert.AreEqual(3, result, "Bind1st");
        }

        [TestMethod]
        public void MakeExpression_ActionTest2()
        {
            var result = default(int);
            var setResult = Make.Action((int i) => result = i);

            var expr = Make.Expression((int a, int b) => setResult(a * b));

            expr.Bind1st(3).Bind1st(4).Compile()();
            Assert.AreEqual(12, result, "Bind1st");

            expr.Bind2nd(4).Bind1st(3).Compile()();
            Assert.AreEqual(12, result, "Bind2nd");
        }

        [TestMethod]
        public void MakeExpression_ActionTest3()
        {
            var result = default(int);
            var setResult = Make.Action((int i) => result = i);

            var expr = Make.Expression((int a, int b, int c) => setResult(a * b % c));

            expr.Bind1st(3).Bind1st(4).Bind1st(7).Compile()();
            Assert.AreEqual(5, result, "Bind1st");
            expr.Bind2nd(4).Bind1st(3).Bind1st(7).Compile()();
            Assert.AreEqual(5, result, "Bind2nd");
            expr.Bind3rd(7).Bind1st(3).Bind1st(4).Compile()();
            Assert.AreEqual(5, result, "Bind3rd");
        }

        [TestMethod]
        public void MakeExpression_ActionTest4()
        {
            var result = default(int);
            var setResult = Make.Action((int i) => result = i);

            var expr = Make.Expression((int a, int b, int c, int d) => setResult(a * b % c + d) );

            expr.Bind1st(3).Bind1st(4).Bind1st(7).Bind1st(1).Compile()();
            Assert.AreEqual(6, result, "Bind1st");
            expr.Bind2nd(4).Bind1st(3).Bind1st(7).Bind1st(1).Compile()();
            Assert.AreEqual(6, result, "Bind2nd");
            expr.Bind3rd(7).Bind1st(3).Bind1st(4).Bind1st(1).Compile()();
            Assert.AreEqual(6, result, "Bind3rd");
            expr.Bind4th(1).Bind1st(3).Bind1st(4).Bind1st(7).Compile()();
            Assert.AreEqual(6, result, "Bind4th");
        }

        [TestMethod]
        public void Cycle_Test1()
        {
            Assert.IsTrue(Make.Cycle(1, 2).Take(4).SequenceEqual(1.UpTo(2).Concat(1.UpTo(2))));
            Assert.IsTrue(Make.Cycle(1, 2, 3).Take(6).SequenceEqual(1.UpTo(3).Concat(1.UpTo(3))));
            Assert.IsTrue(Make.Cycle(1, 2, 3, 4).Take(8).SequenceEqual(1.UpTo(4).Concat(1.UpTo(4))));
            Assert.IsTrue(Make.Cycle(1, 2, 3, 4, 5).Take(10).SequenceEqual(1.UpTo(5).Concat(1.UpTo(5))));
        }

        private class DisposeTestObject : IDisposable
        {
            public EventHandler<EventArgs> Disposed { get; set; }

            public void Dispose()
            {
                if (Disposed != null)
                {
                    this.Disposed(this, new EventArgs());
                }
            }
        }

        [TestMethod]
        public void Repeat_Test1()
        {
            var disposed = false;
            var seq = Make.Repeat(() => new DisposeTestObject() { Disposed = (o, e) => { disposed = true; } });
            var arr = seq.Take(3).ToArray();
            Assert.IsTrue(disposed);
        }

        [TestMethod]
        public void Repeat_Test2()
        {
            var seq1 = Make.Repeat(1).Take(3);
            var seq2 = Make.Repeat(() => 1).Take(3);
            var ans = new[] { 1, 1, 1 };
            Assert.IsTrue(seq1.SequenceEqual(ans));
            Assert.IsTrue(seq2.SequenceEqual(ans));

        }
    }
}
