﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Achiral;
using Achiral.Extension;

namespace Achiral.Test
{
    /// <summary>
    /// Summary description for StringTest
    /// </summary>
    [TestClass]
    public class MaybeTest
    {
        public MaybeTest()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext { get; set; }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void MaybeTest_GetDefaultValue_Simple1()
        {
            Assert.AreEqual(new Maybe<int>().GetValueOrDefault(), 0);
            Assert.AreEqual(new Maybe<int>(1).GetValueOrDefault(), 1);
            Assert.AreEqual(new Maybe<int>().GetValueOrDefault(1), 1);
            Assert.AreEqual(new Maybe<int>(1).GetValueOrDefault(2), 1);
        }

        [TestMethod]
        public void MaybeTest_Normalize_Array1()
        {
            var seq = new Maybe<int[]>().Normalize();
            Assert.IsNotNull(seq);
            Assert.AreEqual(seq.Length, 0);
        }

        [TestMethod]
        public void MaybeTest_Normalize_Array2()
        {
            var seq = new Maybe<int[]>(new[] { 1, 2, 3 }).Normalize();
            Assert.IsNotNull(seq);
            Assert.AreEqual(seq.Length, 3);
        }

        [TestMethod]
        public void MaybeTest_Normalize_IEnumerable1()
        {
            var seq = new Maybe<IEnumerable<int>>().Normalize();
            Assert.IsNotNull(seq);
            Assert.AreEqual(seq.Count(), 0);
        }

        [TestMethod]
        public void MaybeTest_Normalize_IEnumerable2()
        {
            var originalSeq = 1.UpTo(10);
            var seq = Make.Maybe(originalSeq)
                          .Normalize();
            Assert.IsNotNull(seq);
            Assert.IsTrue(seq.SequenceEqual(originalSeq));
        }

        [TestMethod]
        public void MaybeTest_Test_Select()
        {
            var stack = new Stack<int?>();
            var source = new int?[] { 0, null, 1, null, 2 };
            var expected = new int?[] { 2, null, 3, null, 4 };
            source.Reverse().ForEach(item => stack.Push(item));

            var getNext = Make.Func<int?>(
                                 () => Make.Maybe(stack)
                                           .Select(s => s.Pop())
                                           .Select(i => i + 2)
                                           .ToNullable()
                          );

            var result = Enumerable.Range(0, 5)
                                   .Select(_ => getNext())
                                   .ToArray();

            Assert.IsTrue( result.SequenceEqual(expected) );
        }

        [TestMethod]
        public void MaybeTest_Test_SelectMany()
        {
            var stack = new Stack<int?>();
            var source = new int?[] { 0, null, 1, null, 2 };
            var expected = new int?[] { 2, null, 3, null, 4 };
            source.Reverse().ForEach(item => stack.Push(item));

            var getNext = Make.Func<int?>(
                                 () => (from s in Make.Maybe(stack)
                                        from i in Make.Maybe(s.Pop())
                                        from r in Make.Maybe(i + 2)
                                        select r).ToNullable()
                          );

            var result = Enumerable.Range(0, 5)
                                   .Select(_ => getNext())
                                   .ToArray();

            Assert.IsTrue(result.SequenceEqual(expected));
        }
    }
}
